@@ -14,6 +14,9 @@
 # import sys
 # sys.path.insert(0, os.path.abspath('.'))
 
+import sphinx_rtd_theme
+
+html_theme = "sphinx_rtd_theme"
 
 # -- Project information -----------------------------------------------------
 
@@ -22,7 +25,7 @@ copyright = '2021, Jesús Cea Avión'
 author = 'Jesús Cea Avión'
 
 # The full version, including alpha/beta/rc tags
-release = '0.2.0'
+release = '0.5.0'
 
 
 # -- General configuration ---------------------------------------------------
@@ -31,6 +34,7 @@ release = '0.2.0'
 # extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
 # ones.
 extensions = [
+    'sphinx_rtd_theme',
 ]
 
 # Add any paths that contain templates here, relative to this directory.
@@ -48,6 +52,7 @@ exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']
 # a list of builtin themes.
 #
 html_theme = 'alabaster'
+html_theme = "sphinx_rtd_theme"
 
 # Add any paths that contain custom static files (such as style sheets) here,
 # relative to this directory. They are copied after the builtin static files,
