@@ -2,7 +2,7 @@
 
 """Add Table of Contents and chapters to audio files"""
 
-__version__ = '0.2.0'
+__version__ = '0.4.0.post1'
 
 import argparse
 import pathlib
@@ -14,6 +14,7 @@ import functools
 import xml.etree.ElementTree
 import subprocess
 import re
+import sys
 
 import markdown
 import mutagen
@@ -24,19 +25,30 @@ import mutagen.oggvorbis
 
 @functools.total_ordering
 class Offset:
-    def __init__(self, hours=0, minutes=0, seconds=0):
+    def __init__(self, hours=0, minutes=0, seconds=0, compact=True):
+        self.compact = compact
         self._hours = hours
         self._minutes = minutes
         self._seconds = seconds
 
     def __repr__(self):
+        return self.pprint()
+
+    def pprint(self, compact=None):
+        if not self._hours:
+            if compact is None:
+                compact = self.compact
+            if compact:
+                return f'{self._minutes:02d}:{self._seconds:02d}'
         return f'{self._hours:02d}:{self._minutes:02d}:{self._seconds:02d}'
 
-    def add(self, seconds):
+    def add(self, seconds, compact=None):
+        if compact is None:
+            compact = self.compact
         offset, seconds = divmod(self._seconds + seconds, 60)
         offset, minutes = divmod(self._minutes + offset, 60)
         hours = self._hours + offset
-        return type(self)(hours, minutes, seconds)
+        return type(self)(hours, minutes, seconds, compact=compact)
 
     def __eq__(self, other):
         if not isinstance(other, type(self)):
@@ -52,30 +64,41 @@ class Offset:
         return self._hours * 3600 + self._minutes * 60 + self._seconds
 
     @classmethod
-    def from_seconds(cls, seconds):
-        return cls().add(seconds)
+    def from_seconds(cls, seconds, compact=True):
+        return cls(compact=compact).add(seconds)
 
 
 class Timestamps(markdown.inlinepatterns.InlineProcessor):
-    def __init__(self, *args, **kwargs):
+    # Beside default value, explicing "offset" takes it out of
+    # both "args" and "kwargs". Useful when calling "super()__init__()".
+    def __init__(self, *args, offset=0, **kwargs):
         super().__init__(*args, **kwargs)
+        self.offset = offset
 
     def handleMatch(self, m, data):
         hour = m.group('hours')
+        compact = hour is None
         hour = 0 if hour is None else int(hour)
         timestamp = Offset(hours=hour,
                            minutes=int(m.group('minutes')),
-                           seconds=int(m.group('seconds')))
+                           seconds=int(m.group('seconds')),
+                           compact=compact)
+        timestamp = timestamp.add(self.offset)
         ts = timestamp.to_seconds()
         # We can not simply set "text" because in a Element tree,
         # any text in a tag goes BEFORE any tag child.
         # https://docs.python.org/3/library/
         #         xml.etree.elementtree.html#xml.etree.ElementTree.Element.text
 
-        el = etree.Element('timestamp', attrib={'ts': str(ts)})
+        attrib = {
+                    'ts': str(ts),
+                    'compact': str(int(timestamp.compact)),
+                    'offset': str(self.offset),
+                 }
+        el = etree.Element('timestamp', attrib=attrib)
         el2 = etree.Element('strong')
         # To avoid infinite recursion, "[]" reintroduced at postprocessing
-        el2.text = f'[!*!{m.group(1)[1:]}'
+        el2.text = f'[!*!{timestamp.pprint()}]'
         el3 = etree.Element('topic')
         el3.text = data[m.end(0):]
         el.extend((el2, el3))
@@ -83,26 +106,27 @@ class Timestamps(markdown.inlinepatterns.InlineProcessor):
 
 
 class TimestampsExtension(markdown.extensions.Extension):
+    # Beside default value, explicing "offset" takes it out of
+    # both "args" and "kwargs". Useful when calling "super()__init__()".
+    def __init__(self, *args, offset=0, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.offset = offset
+
     def extendMarkdown(self, md):
         pattern = (r'(\[((?P<hours>[0-9]?[0-9]):|)'
                    r'(?P<minutes>[0-9]?[0-9]):(?P<seconds>[0-9][0-9])\])')
-        md.inlinePatterns.register(Timestamps(pattern, md),
+        md.inlinePatterns.register(Timestamps(pattern, md, offset=self.offset),
                                    'timestamps', 10000)
 
 
 class Toc:
-    def __init__(self, file):
+    def __init__(self, file, offset=0):
         if isinstance(file, pathlib.Path):
             file = file.read_text(encoding='utf-8')
 
         self.text = file
 
-        title = self.text.split(sep='\n', maxsplit=1)[0]
-        while title[0] == '#':
-            title = title[1:]
-        self.title = title.strip()
-
-        md = markdown.Markdown(extensions=[TimestampsExtension()])
+        md = markdown.Markdown(extensions=[TimestampsExtension(offset=offset)])
         self.html = md.convert(self.text).replace('[!*!', '[')
 
         # We get the timestamps as a postprocessing step instead of doing it
@@ -114,7 +138,8 @@ class Toc:
         timestamps = []
         last = Offset().add(-1)  # Negative number is API *ABUSE*!
         for timestamp in root.findall('.//timestamp'):
-            offset = Offset().add(int(timestamp.get('ts')))
+            compact = bool(int(timestamp.get('compact')))
+            offset = Offset(compact=compact).add(int(timestamp.get('ts')))
             if last >= offset:
                 raise ValueError('Non monotonic timestamps: '
                                  f'{last} > {offset}')
@@ -130,15 +155,21 @@ class Toc:
         self.timestamps = timestamps
 
 
-def show_in_browser(toc):
+def show_in_browser(toc, msg=None):
     fd, tmpname = tempfile.mkstemp(suffix='.html')
+    print()
+    print(toc.html)
+    print()
     try:
         html = '<html><head><meta charset="utf-8"/></head><body>'
         html = f'{html}{toc.html}</body></html>'
         os.write(fd, html.encode('utf-8'))
         os.close(fd)
         webbrowser.open('file://' + tmpname)
-        print('Press ENTER to continue')
+
+        if msg:
+            print(msg, file=sys.stderr)
+        print('Press ENTER to continue', file=sys.stderr)
         input()
     finally:
         os.unlink(tmpname)
@@ -151,7 +182,7 @@ def show_in_browser(toc):
 
 def add_tags_mp4(path, toc, add_toc=False, add_chapters=False):
     if add_chapters:
-        metadata = f';FFMETADATA1\ntitle={toc.title}\n'
+        metadata = f';FFMETADATA1\n'
         for ts_start, ts_end, title in toc.timestamps:
             metadata += '\n[CHAPTER]\nTIMEBASE=1/1000\n'
             metadata += f'START={ts_start.to_seconds()*1000}\n'
@@ -191,7 +222,7 @@ def add_tags_ogg(ogg, toc, add_toc=False, add_chapters=False):
 
         for i, (ts_start, ts_end, title) in enumerate(toc.timestamps):
             chapter = f'CHAPTER{i:03}'
-            ogg.tags[chapter] = repr(ts_start)
+            ogg.tags[chapter] = ts_start.pprint(compact=False)
             ogg.tags[chapter+'NAME'] = title
         ogg.save()
 
@@ -209,19 +240,23 @@ def add_tags_vorbis(path, toc, add_toc=False, add_chapters=False):
 def add_tags_mp3(path, toc, add_toc=False, add_chapters=False):
     try:
         tags = mutagen.id3.ID3(path)
-
-        # Delete existing TOC
-        ctoc = tags.get('CTOC:toc')
-        if ctoc:
-            for i in ctoc.child_element_ids:
-                del tags[f'CHAP:{i}']
-            del tags['CTOC:toc']
     except mutagen.id3.ID3NoHeaderError:
         tags = mutagen.id3.ID3()
 
+    title = tags.get('TIT2', '')
+    if title:
+        title = title.text
+
+    # Delete existing TOC
+    ctoc = tags.get('CTOC:toc')
+    if ctoc:
+        for i in ctoc.child_element_ids:
+            del tags[f'CHAP:{i}']
+        del tags['CTOC:toc']
+
     if add_chapters:
         # XXX: Change this
-        title = mutagen.id3.TIT2(text=[toc.title])
+        title = mutagen.id3.TIT2(text=title)
         chapters = [f'chp{i}' for i in range(1, len(toc.timestamps) + 1)]
         tags.add(
             mutagen.id3.CTOC(element_id='toc',
@@ -259,27 +294,57 @@ def add_tags_audio(audios, toc, add_toc, add_chapters):
 def main():
     parser = argparse.ArgumentParser(
             description='Add Table of Contents and chapters to audio files')
+    parser.add_argument('--version', action='store_true',
+                        help='Show version info')
+    parser.add_argument('--offset', nargs=1,
+                        help='Seconds or [HH:]MM:SS to add to ALL timestamps')
     parser.add_argument('--show', action='store_true',
                         help='Show the generated HTML in your browser')
     parser.add_argument('--toc', action='store_true',
                         help='Store Table of Contents in the audio file')
     parser.add_argument('--chapters', action='store_true',
                         help='Store chapters details in the audio file')
-    parser.add_argument('TOC', nargs=1,
+    parser.add_argument('TOC', nargs='?',
                         help='Table of Contents file')
     parser.add_argument('AUDIO', nargs='*', type=pathlib.Path,
                         help='Audio file')
 
     args = parser.parse_args()
+    if args.version:
+        print(__version__)
+        sys.exit()
+    offset = args.offset
+    if offset is None:
+        offset = 0
+    else:
+        value = offset[0]
+        if ':' in value:
+            offset = 0
+            for n in value.split(':'):
+                offset = offset * 60 + int(n)
+        else:
+            offset = int(value)
+
+    toc = args.TOC
+    if toc is None:
+        parser.error('TOC argument required')
+
+    if args.toc:
+        parser.error('"--toc" parameter is not YET supported')
+
     if (args.toc or args.chapters) and (not args.AUDIO):
         # XXX: Undocumented
         parser.error('AUDIO arguments required')
-    toc = args.TOC[0]
 
-    toc = Toc(pathlib.Path(toc))
+    toc = Toc(pathlib.Path(toc), offset=offset)
+    offset_hh_mm_ss = Offset().add(offset).pprint()
+    msg = (f'We applied an offset of {offset} seconds ({offset_hh_mm_ss}) '
+           'to all timestamps')
 
     if args.show:
-        show_in_browser(toc)
+        show_in_browser(toc, msg=msg)
+    else:
+        print(msg, file=sys.stderr)
 
     if args.toc or args.chapters:
         add_tags_audio(args.AUDIO, toc,
